import time
import argparse
import numpy as np
import rpy2.robjects as ro
from utils import *


parser = argparse.ArgumentParser(description="Webserver TSFinder prediction")
parser.add_argument('-i', '--input_dir', default='./', type=str,
                    help='Path to processed data directory')
parser.add_argument('-o', '--output_dir', default='./', type=str,
                    help='Path to target data directory')
parser.add_argument('-t', '--tissue_name', default='lung', type=str,
                    help='The selected tissue name')
parser.add_argument('-c', '--checkpoint_dir', default='/home/bowen.song/m6acsvar/58web/tissue/', type=str,
                    help='Path to checkpoint data directory')
parser.add_argument('-j', '--job_id', default=1, type=int,
                    help='Job ID')
parser.add_argument('-l', '--inst_len', default=40, type=int,
                    help='Instance length')
parser.add_argument('-s', '--inst_stride', default=5, type=int,
                    help='Instance stride')
parser.add_argument('-thres', '--threshold', default=None, type=int, choices=range(0, 101),
                    metavar="[0-100]",
                    help='Threshold (0-100) denoting at what threat level to predict as positive.')
args = parser.parse_args()

readRDS = ro.r['readRDS']

print('Load data from:', args.input_dir)
example_token = readRDS(args.input_dir + 'file_token.rds')
example_token = np.asarray(example_token)

model = WeakRM()
model(np.ones((1, 1, args.inst_len, 4)).astype(np.float32))
# it is assumed that the saved model for each tissue is saved in one dir under a checkpoint dir
checkpoint_filepath = args.checkpoint_dir + args.tissue_name + '/WeakRM_base.h5'
print('Load weights from:', checkpoint_filepath)
model.load_weights(checkpoint_filepath)

example_bags = []
for seq in example_token:
    one_hot_bag = embed(seq, args.inst_len, args.inst_stride)
    example_bags.append(one_hot_bag)

print('Prediction start!')
predictions = []

for idx, tdata in enumerate(example_bags):
    start_time = time.time()
    pred, _ = model(tdata.reshape(1, -1, args.inst_len, 4), training=False)
    predictions.append(pred.numpy())
    print("{}/{} finished! Time costs: {}s.".format(
        int(idx + 1), len(example_bags), np.round(time.time() - start_time, 2)))

predictions = np.concatenate(predictions, axis=0)
result = predictions.reshape(1, -1)

# if threshold is given, the result will be converted to a binary label
if args.threshold:
    result = result > (args.threshold / 100)

print('Prediction results are saving to: ', args.output_dir)
np.savetxt(args.output_dir + 'result.txt', result, delimiter='\t')

