import sys
sys.path.append('/opt/anaconda/anaconda3/envs/m6acsvar/lib/')
import numpy as np
import tensorflow as tf
from tensorflow.keras.regularizers import l2

tfk = tf.keras
tfkl = tf.keras.layers


class WeakRM(tf.keras.Model):

    def __init__(self):
        super(WeakRM, self).__init__()

        self.conv1 = tfkl.Conv1D(32, 15, padding='same', activation='relu')
        self.conv2 = tfkl.Conv1D(16, 5, padding='same', activation='relu',
                                 kernel_regularizer=l2(0.005))
        self.dropout = tfkl.Dropout(0.25)
        self.pool1 = tfkl.MaxPool1D(pool_size=2)

        self.att_v = tfkl.Dense(128, activation='tanh')
        self.att_u = tfkl.Dense(128, activation='sigmoid')

        self.attention_weights = tfkl.Dense(1)

        self.classifier = tfkl.Dense(1, activation='sigmoid')

    def call(self, inputs, training=True, mask=None):
        input_bag = tf.squeeze(inputs, axis=0)

        inst_conv1 = self.conv1(input_bag)
        inst_pool1 = self.pool1(inst_conv1)
        inst_pool1 = self.dropout(inst_pool1, training=training)

        inst_conv2 = self.conv2(inst_pool1)

        inst_features = tfkl.Flatten()(inst_conv2)

        attention_vmatrix = self.att_v(inst_features)
        attention_umatrix = self.att_u(inst_features)

        gated_attention = self.attention_weights(attention_vmatrix * attention_umatrix)

        gated_attention = tf.transpose(gated_attention, perm=[1, 0])
        gated_attention = tfkl.Softmax()(gated_attention)

        bag_features = tf.matmul(gated_attention, inst_features)

        bag_probability = self.classifier(bag_features)

        return bag_probability, gated_attention


def embed(sequence, instance_len, instance_stride):
    instance_num = int((len(sequence)-instance_len)/instance_stride)+1
    bag = []
    for i in range(instance_num):
        instance = sequence[i*instance_stride:i*instance_stride+instance_len]
        bag.append(instance)
    bag = np.stack(bag).astype(np.int32)
    one_hot_bag = np.eye(4)[bag-1].astype(np.float32)
    return one_hot_bag


def embed_backup(sequence, instance_len, instance_stride, zero_pad=True):
    instance_num = (len(sequence)-instance_len)/instance_stride + 1
    if len(sequence) < instance_len:
        gap = instance_len - len(sequence)
    else:
        gap = int((max(-1, np.floor(instance_num-1)) +
                 (np.ceil(instance_num) - np.floor(instance_num))) *
                 instance_stride + instance_len - len(sequence))
    left_pad = int(gap / 2)
    right_pad = gap - left_pad
    padded_seq = np.pad(sequence, (left_pad, right_pad), 'constant', constant_values=5)
    bag = []
    for i in np.arange(max(1, np.floor(instance_num)+(gap > 0))):
        instance = padded_seq[int(i*instance_stride):int(i*instance_stride+instance_len)]
        bag.append(instance)
    bag = np.stack(bag).astype(np.int32)

    one_hot_bag = np.eye(5)[bag-1][:, :, :4].astype(np.float32)
    condition = (bag == 5).reshape(-1, instance_len, 1)
    if zero_pad:
        one_hot_bag = np.where(condition, np.zeros_like(one_hot_bag, dtype=np.float32), one_hot_bag)
    else:
        one_hot_bag = np.where(condition, np.ones_like(one_hot_bag, dtype=np.float32)*(1/4), one_hot_bag)
    return one_hot_bag.astype(np.float32)
